import csv
import DataRow
import codecs
import sys


class LoadApacheLogData:
    """ A class for loading apache log data into a data row. """

    rows = []
    rowCount = 0

    limitLoad = True
    rowLimit = 25

    cleanData = False
    cleanCount = 0

    verbose = False
    ignoreHeader = True
    header = ['Ip1', 'Ip2', 'Ip3', 'Ip4', 'HttpVersion', 'Timestamp', 'Url', 'ResponseCode', 'Length', 'Client', 'Answer']

    def __init__(self, lRows=[], lLimitLoad=False, lRowLimit=-1, lCleanData=False, lVerbose=False):
        self.rows = lRows
        self.limitLoad = lLimitLoad
        self.rowLimit = lRowLimit
        self.cleanData = lCleanData
        self.verbose = lVerbose
    #edef

    '''
    1 - - 127.0.0.1 - - [timeStamp] "POST url HTTP/1.1" 200 957 "-" "Apache-HttpClient/4.5.2 (Java/1.8.0_91)"
    
    answer: 1
    ip: 127.0.0.1
    opt1: -
    opt2: -
    timeStamp: []
    url: "POST url HTTP/1.1"
    responseCode: 200
    length: 957
    opt3: "-"
    client: "Apache-HttpClient/4.5.2 (Java/1.8.0_91)"
    '''

    #alf = apache log file
    def loadData(self, logFile='', type='alf', version='1.0', reset=False, dataMap={}, appendCols={}):
        print ("")
        print ("")
        print("Loading Data: " + logFile + " Type: " + type + " Version: " + version + " Reset: " + str(reset))

        if self.verbose:
            print ("Found data mapping:")
            for i in dataMap:
                print(i, dataMap[i])
            #efl
        #eif

        if self.verbose:
            print ("Found append cols mapping:")
            for i in appendCols:
                print(i)
            #efl
        #eif

        if reset == True:
            print ("Resetting rows:")
            self.resetRows()
        #eif

        if type == 'alf' and version == '1.0' and logFile != '':
            ifile = codecs.open(logFile, 'rb', encoding="utf-8-sig")
            reader = ifile.readlines()

            lrows = []
            rownum = 0

            for row in reader:
                if rownum == 0 and self.ignoreHeader == False:
                    header = row
                    rownum += 1
                else:
                    colnum = 0
                    dRow = DataRow.DataRow()
                    dRow.verbose = self.verbose

                    # Append static values outside of the csv like stock symbol etc
                    for entry in appendCols:
                        for key in entry:
                            dRow.setMember(key, str(entry['idx']), entry[key])
                        # efl
                    # efl

                    if self.verbose:
                        print ('')
                    # eif

                    rowCols = self.processRow(row)
                    if len(rowCols) > 0:
                        for col in rowCols:
                            if self.verbose:
                                print (' %-8s: %s' % (self.header[colnum], col))
                            #eif

                            colName = self.header[colnum]
                            if len(dataMap) > 0:
                                try:
                                    memberIdx = dataMap[colName]
                                    if int(memberIdx) != -1:
                                        dRow.setMember(colName, int(memberIdx), col)
                                    # eif
                                except:
                                    if self.verbose:
                                        print ("Error setting member with index: ", colnum, " with value: ", col)
                                        print ("Unexpected error:", sys.exc_info()[0])
                                    # eif
                                # etry

                            else:
                                dRow.setMember(colName, colnum, col)

                            # eif

                            colnum += 1
                        # efl

                        if self.verbose:
                            if self.limitLoad:
                              dRow.printDataRow()
                            # eif
                        # eif

                        dRow.stampId()
                        lrows.append(dRow.copy())
                        self.rowCount += 1
                    # eif

                    rownum += 1
                # eif

                if self.limitLoad == True and self.rowCount >= self.rowLimit and self.rowLimit > 0:
                    break;
                # eif

            # efl

            ifile.close()

            print ("Loaded %i rows from this data file." % (rownum))
            lrows = self.sortRows(lrows)
            self.cleanRows(lrows)
            self.rows.extend(lrows)

            print ('CleanCount: %i RowCount: %i RowsFound: %i' % (self.cleanCount, self.rowCount, len(self.rows)))
        # eif
    # edef

    def processRow(self, row):
        # 127.0.0.1 - - [21/Sep/2016:13:58:06 +0000] "POST url HTTP/1.1" 200 957 "-" "Apache-HttpClient/4.5.2 (Java/1.8.0_91)"
        # print("Row: %s" % (row))

        if row == None or row == "":
            return []
        # eif

        nr1 = row.split(" - - ")

        if len(nr1) == 2:
            answer = -1
            ip = nr1[0]
            nr2 = nr1[1]
        else:
            answer = nr1[0]
            ip = nr1[1]
            nr2 = nr1[2]
        # eif

        nr3 = nr2.split(" \"-\" ")

        if len(nr3) != 2:
            return []
        # eif

        client = nr3[1].replace("\"", "")
        client = client.replace("/", "_")
        nr4 = nr3[0]
        nr4 = nr4.replace("[", "")
        nr4 = nr4.replace("]", "")
        nr4 = nr4.replace("\"", "")

        # 21/Sep/2016:13:58:06 +0000 POST url HTTP/1.1 200 957
        nr5 = nr4.split(" ")

        if len(nr5) != 7:
            return []
        # eif

        date = nr5[0]
        offset = nr5[1]
        httpCmd = nr5[2]
        url = nr5[3].replace("/", "_")
        httpVer = nr5[4].replace("/", "_")
        httpResCode = nr5[5]
        httpLen = nr5[6]

        '''
        print ('Client: %s' % (client))
        print ('Ip: %s' % (ip))
        print ('Date: %s' % (date))
        print ('Offset: %s' % (offset))
        print ('HttpCmd: %s' % (httpCmd))
        print ('Url: %s' % (url))
        print ('HttpVer: %s' % (httpVer))
        print ('HttpResCode: %s' % (httpResCode))
        print ('HttpLen: %s' % (httpLen))
        '''
        ips = ip.split(".")
        ip1 = ips[0]
        ip2 = ips[1]
        ip3 = ips[2]
        ip4 = ips[3]

        return [ip1, ip2, ip3, ip4, httpVer, (date + " " + offset), url, httpResCode, httpLen, client, answer]
    # edef

    def resetRows(self):
        self.rows = []
        self.rowCount = 0
        self.cleanCount = 0
    # edef

    def sortRows(self, lrows):
        return sorted(lrows, key=id)
    # edef

    def cleanRows(self, lrows):
        if self.cleanData == True:
            print ("Cleaning row data...")
            should_restart = True
            while should_restart:
                should_restart = False
                for row in lrows:
                    if row.error == True:
                        lrows.remove(row)
                        self.rowCount -= 1
                        self.cleanCount += 1
                        should_restart = True
                    # eif
                # efl
            # fwl
        # eif
    # edef

# eclass