import DataRow
import LoadCsvData
import sys

class LoadFeatureData:
    """ A class for adding calculated features to the data rows loaded from csv. """

    rows = []
    rowCount = 0

    limitLoad = True
    rowLimit = 25

    cleanData = False
    cleanCount = 0

    verbose = False
    loadCsvData = None
    extras = []

    def __init__(self, lLimitLoad=False, lRowLimit=-1, lCleanData=False, lVerbose=False, lLoadCsvData=None):
        self.limitLoad = lLimitLoad
        self.rowLimit = lRowLimit
        self.cleanData = lCleanData
        self.verbose = lVerbose
        self.loadCsvData = lLoadCsvData
    # edef

    def setExtras(self, extrs):
        self.extras = extrs
    # edef

    def generateData(self, type='goog_stock_sma100'):
        print ("")
        print ("")
        print("Generating Feature Data: Type: " + type)

        if type == '':
            # Assumed that an answer column has been imported and is not expected to be calculated.
            self.rowCount = self.loadCsvData.rowCount
            rownum = self.rowCount

            lrows = []
            lrows.extend(self.loadCsvData.rows)

            print ("Loaded %i rows from this data file." % (rownum))
            lrows = self.sortRows(lrows)
            self.cleanRows(lrows)
            self.rows.extend(lrows)
            print ('CleanCount: %i RowCount: %i RowsFound: %i' % (self.cleanCount, self.rowCount, len(self.rows)))

        elif type == 'weight_age':
            self.rowCount = self.loadCsvData.rowCount
            rownum = self.rowCount

            lrows = []
            lrows.extend(self.loadCsvData.rows)

            for i in xrange(len(lrows)):
                # The answer in linear regression models will always be stored in the 'Answer' column
                lrows[i].setMember('Answer', 3, float(lrows[i].getMemberByName('BloodFat')))
            # efl

            print ("Loaded %i rows from this data file." % (rownum))
            lrows = self.sortRows(lrows)
            self.cleanRows(lrows)
            self.rows.extend(lrows)
            print ('CleanCount: %i RowCount: %i RowsFound: %i' % (self.cleanCount, self.rowCount, len(self.rows)))

        elif type == 'goog_stock_sma100':
            self.resetRows()

            lrows = []
            lrows.extend(self.loadCsvData.rows)
            rownum = 0
            avg = 0
            target = 100
            cnt = 0
            startingVal = lrows[0].getMemberByName('Close')

            for i in xrange(len(lrows)):
                try:
                    float(lrows[i].getMemberByName('Close'))
                    float(lrows[i].getMemberByName('Open'))
                    float(lrows[i].getMemberByName('High'))
                    float(lrows[i].getMemberByName('Low'))
                except:
                    lrows[i].error = True
            # efl
            self.cleanRows(lrows)

            for i in xrange(len(lrows)):
                # The answer in linear regression models will always be stored in the 'Answer' column
                lrows[i].setMember('Answer', 8, float(lrows[i].getMemberByName('Close')))

                if i < target:
                    lrows[i].setMember('sma_100', 7, startingVal)
                else:
                    avg = 0

                    for j in xrange(len(lrows)):
                        if j < (i - target):
                            ''' do nothing '''
                        else:
                            if j >= (i - target) and j <= (i - 0):
                                avg += float(lrows[j].getMemberByName('Close'))
                            elif j > (i - 0):
                                break
                            # eif
                        # eif
                    # efl

                    if self.verbose == True:
                        print('sma_100: %d' % (float(avg) / float(target)))
                    # eif

                    lrows[i].setMember('sma_100', 7, (float(avg) / float(target)))
                # eif

                if self.limitLoad == True and cnt >= self.rowLimit and self.rowLimit > 0:
                    break;
                # eif

                rownum += 1
                cnt += 1
            # efl

            print ("Loaded %i rows from this data file." % (rownum))
            lrows = self.sortRows(lrows)
            self.cleanRows(lrows)
            self.rows.extend(lrows)
            self.rowCount = len(lrows)

            print ('CleanCount: %i RowCount: %i RowsFound: %i' % (self.cleanCount, self.rowCount, len(self.rows)))

        elif type == 'goog_log_reg':
            self.resetRows()

            lrows = []
            lrows.extend(self.loadCsvData.rows)
            rownum = 0
            length = len(lrows)

            for i in xrange(length):
                try:
                    float(lrows[i].getMemberByName('Close'))
                    float(lrows[i].getMemberByName('Open'))
                    float(lrows[i].getMemberByName('High'))
                    float(lrows[i].getMemberByName('Low'))

                    if i + 1 < length:
                        if float(lrows[i + 1].getMemberByName('Close')) > float(lrows[i].getMemberByName('Close')):
                            lrows[i].setMember('Answer', 7, 1.00)
                        else:
                            lrows[i].setMember('Answer', 7, 0.00)
                        # eif
                    else:
                        lrows[i].setMember('Answer', 7, 0.00)
                    # eif

                except:
                    lrows[i].error = True
                    lrows[i].setMember('Answer', 7, 0.00)

                # etry

                rownum = rownum + 1
            # efl

            print ("Loaded %i rows from this data file." % (rownum))
            lrows = self.sortRows(lrows)
            self.cleanRows(lrows)
            self.rows.extend(lrows)
            self.rowCount = len(lrows)

            print ('CleanCount: %i RowCount: %i RowsFound: %i' % (self.cleanCount, self.rowCount, len(self.rows)))

        elif type == 'blue_ice_log_reg':
            self.resetRows()

            lrows = []
            lrows.extend(self.loadCsvData.rows)
            rownum = 0
            length = len(lrows)

            for i in xrange(length):
                try:
                    float(lrows[i].getMemberByName('Ip1'))
                    float(lrows[i].getMemberByName('Ip2'))
                    float(lrows[i].getMemberByName('Ip3'))
                    float(lrows[i].getMemberByName('Ip4'))
                    float(lrows[i].getMemberByName('ResponseCode'))
                    float(lrows[i].getMemberByName('Answer'))

                    llen = lrows[i].getMemberByName('Length')
                    float(llen)

                    if llen == 0:
                        lrows[i].setMember('IsZeroLen', 12, 2.00)
                    else:
                        lrows[i].setMember('IsZeroLen', 12, 1.00)
                    # eif

                    url = lrows[i].getMemberByName('Url')
                    lfound = False
                    for r in self.extras:
                        if url == "_" or r in url:
                            lrows[i].setMember('IsUrlKnown', 11, 2.00)
                            lfound = True
                            break
                        # eif

                    # efl

                    if lfound == False:
                        lrows[i].setMember('IsUrlKnown', 11, 1.00)
                    # eif

                    if float(lrows[i].getMemberByName('Answer')) == 1.0:
                        lrows[i].setMember('AnswerCatYes', 13, 2.00)
                        lrows[i].setMember('AnswerCatNo', 14, 1.00)
                    elif float(lrows[i].getMemberByName('Answer')) == -1.0:
                        lrows[i].setMember('AnswerCatYes', 13, 0.00)
                        lrows[i].setMember('AnswerCatNo', 14, 0.00)
                    else:
                        lrows[i].setMember('AnswerCatYes', 13, 1.00)
                        lrows[i].setMember('AnswerCatNo', 14, 2.00)
                    # eif

                    if self.verbose:
                        print("ResponseCode: %s" % (lrows[i].getMemberByName('ResponseCode')))
                        print("Answer: %s" % (lrows[i].getMemberByName('Answer')))
                        print("Url: %s" % (lrows[i].getMemberByName('Url')))
                        print("IsUrlKnown: %s" % (lrows[i].getMemberByName('IsUrlKnown')))
                        print("AnswerCatYes: %s" % (lrows[i].getMemberByName('AnswerCatYes')))
                        print("AnswerCatNo: %s" % (lrows[i].getMemberByName('AnswerCatNo')))
                    # eif
                except:
                    print ("Unexpected error: %s" % sys.exc_info()[0])
                    lrows[i].error = True
                    raise

                # etry

                rownum = rownum + 1
            # efl

            print ("Loaded %i rows from this data file." % (rownum))
            lrows = self.sortRows(lrows)
            self.cleanRows(lrows)
            self.rows.extend(lrows)
            self.rowCount = len(lrows)

            print ('CleanCount: %i RowCount: %i RowsFound: %i' % (self.cleanCount, self.rowCount, len(self.rows)))

            # eif

        # eif
    # edef

    def resetRows(self):
        self.rows = []
        self.rowCount = 0
        self.cleanCount = 0
    # edef

    def sortRows(self, lrows):
        return sorted(lrows, key=id)
    # edef

    def cleanRows(self, lrows):
        if self.cleanData == True:
            print ("Cleaning row data...")
            should_restart = True
            while should_restart:
                should_restart = False
                for row in lrows:
                    if row.error == True:
                        lrows.remove(row)
                        self.rowCount -= 1
                        self.cleanCount += 1
                        should_restart = True
                    # eif
                # efl
            # fwl
        # eif
    # edef

# eclass
